#ifndef PROTOCOL_H
#define PROTOCOL_H

typedef unsigned int	u32;
typedef int		s32;
typedef unsigned short	u16;
typedef short		s16;
typedef unsigned char	u8;
typedef char		s8;

#define STR_DUMP_HW		"%02X:%02X:%02X:%02X:%02X:%02X"
#define VAR_DUMP_HW(hw)		(hw)[0],(hw)[1],(hw)[2],(hw)[3],(hw)[4],(hw)[5]
#define STR_DUMP_IP		"%d.%d.%d.%d"
#define VAR_DUMP_IP(ip)		((u8*)ip)[0],((u8*)ip)[1],((u8*)ip)[2],((u8*)ip)[3]

#define ICMP_ECHOREPLY		0	/* Echo Reply                   */
#define ICMP_DEST_UNREACH	3	/* Destination Unreachable      */
#define ICMP_SOURCE_QUENCH	4	/* Source Quench                */
#define ICMP_REDIRECT		5	/* Redirect (change route)      */
#define ICMP_ECHO		8	/* Echo Request                 */
#define ICMP_TIME_EXCEEDED	11	/* Time Exceeded                */
#define ICMP_PARAMETERPROB	12	/* Parameter Problem            */
#define ICMP_TIMESTAMP		13	/* Timestamp Request            */
#define ICMP_TIMESTAMPREPLY	14	/* Timestamp Reply              */
#define ICMP_INFO_REQUEST	15	/* Information Request          */
#define ICMP_INFO_REPLY		16	/* Information Reply            */
#define ICMP_ADDRESS		17	/* Address Mask Request         */
#define ICMP_ADDRESSREPLY	18	/* Address Mask Reply           */

#define WINDOWSIZE 	4096
#define TSENDSIZE 	512
#define DEFWINDOW 	1024
#define DEFSEG		1024
#define TMAXSIZE 	1024
#define UMAXLEN 	1024
#define ICMPMAX 	300

/* protocol type */
#define PROTUDP		17
#define PROTTCP		6		/* standard protocol types for IP */
#define PROTICMP	1

/* packet type */
#define  EIP     0x0008
#define  EARP    0x0608
#define  ERARP	 0x3580		/* I guess this is RARP */
#define  ECHAOS  0x0408
#define  EIPX  0x3781		/* ipx */

/* ethernet header */
struct eth
{
  u8 dst[6];		/* packet destination */
  u8 src[6];		/* packet source */
  u16 type;			/* packet type  */
};

/*#define IPHEADER_SIZE		20*/

/* ip header */
struct iph
{
  u8 ihl:4, version:4;
  u8 service;			/* type of service for IP */
  u16 tot_len;			/* total length of IP packet */
  u16 id;			/* these are all BYTE-SWAPPED! */
  u16 frags;			/* combination of flags and value */
  u8 ttl;			/* time to live */
  u8 protocol;			/* higher level protocol type */
/* header size = ten bytes */
  u16 check;			/* header checksum, byte-swapped */
  u8 src[4];			/* source IP */
  u8 dst[4];			/* dest IP */

  u8 data[0];			/* data 'mark' */
};

/* ip - icmp data part */ 
  struct icmp
{
  
u8 type;			/* ICMP type field */
  
u8 code;			/* ICMP code field */
  
u16 check;			/* checksum */
  
u16 part1;			/* depends on type and code */
  
u16 part2;

  u8 data[0];
 
};

#define RARPR	0x0400		  	/*  RARP reply, from host */
#define RARPQ	0x0300			/*  RARP request */
#define ARPREP  0x0200		  	/*  reply */
#define ARPREQ  0x0100		  	/*  request */
#define ARPPRO	0x0008			/*  IP protocol */

#define HTYPE 	0x0100			/*  Ethernet hardware type */
  struct arp_pkt
{
  
u16 hrd;			/* hardware type, Ethernet = 1 */
  
u16 pro;			/* protocol type to resolve for */
  
u8 hln;			/* byte length of hardware addr = 6 for ETNET */
  
u8 pln;			/* byte length of protocol = 4 for IP */
  
u16 op;			/* opcode, request = 1, reply = 2, RARP = 3,4 */
  
u8 s_hw[6];		/* source/sender hw address */
  u8 s_ip[4];		/* source/sender ip address */
  u8 d_hw[6];		/* dest. hw */
  u8 d_ip[4];
   
				/* dest. ip */
};


struct arp_cache
{
  
u8 hw[6];		/* hw address */
  
u8 ip[4];		/* ip address */
  
u8 fl_used;			/* used or not */
  
u32 tm;			/* time of last access */

};


struct udph
{
  u16 src_port;			/* source port */
  u16 dst_port;			/* destination port */
  u16 length;			/* length of packet including header */
  u16 check;			/* tcp checksum */
};

/* pseudo-tcp head -- used for checksumming */
struct ptcph
{
  u8 src[4];
  u8 dst[4];
  u8 zero;
  u8 proto;
  u16 len;
};

#define DEFAULT_TCPH_LEN	0x50
#define GET_TCPH_LEN(hlen)	(((hlen)&0xF0)>>2)

struct tcph
{
  u16 src_port;
  u16 dst_port;
  u32 seq;
  u32 ack;
  u8 hlen;
  u8 flags;
  u16 window;
  u16 check;
  u16 urgent;
};

#endif
