#include <sys/socket.h>
#include <string.h>
#include <stdio.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <signal.h>
#include <net/if.h>
#include <netinet/in.h>
#include <errno.h>
#include <getopt.h>
#include <stdarg.h>
#include <time.h>
#include "protocol.h"

#define MAXDEST		16
int nr_dests = 0;
int dests[MAXDEST];
char *ifnames[MAXDEST] = { NULL, };

void
do_send (int skt, char *buff, int bts, struct sockaddr_ll sa, int ifidx)
{
  int ret;
  int ifnr = dests[ifidx];
  int sa_len = sizeof (sa);

  printf ("  ==> %s", ifnames[ifidx]);
  sa.sll_ifindex = ifnr;

  if ((ret = sendto (skt, buff, bts, 0, (struct sockaddr *) &sa, sa_len)) < 0)
    {
      putchar ('\n');
      perror ("sendto");
    }
  else
    printf (" (%d of %d bytes sent)\n", ret, bts);
}

void
distribute (int skt, char *buff, int bts, struct sockaddr_ll from_ll)
{
  struct sockaddr_ll sa;
  int sa_len = sizeof (sa);
  struct iph *iph;
  struct udph *udph;
  int i, j;

  iph = (struct iph *) buff;
  udph = (struct udph *) ((void *) iph + sizeof (struct iph));

  for (i = 0; i < nr_dests; i++)
    {
      if (dests[i] == from_ll.sll_ifindex)
	break;
    }

  if (i == nr_dests)
    return;			/* ignored interface */

  printf ("%s: " STR_DUMP_IP ":%d ==> " STR_DUMP_IP ":%d  (%d bytes)\n",
	  ifnames[i],
	  VAR_DUMP_IP (iph->src), ntohs (udph->src_port),
	  VAR_DUMP_IP (iph->dst), ntohs (udph->dst_port), bts);

  {
    int x;
    for (x = 0; x < 8; x++)
      from_ll.sll_addr[x] = 0xff;

    //  printf("%d ", from_ll.sll_addr[x]);
    // putchar('\n');
  }

  for (j = 0; j < i; j++)
    do_send (skt, buff, bts, from_ll, j);

  for (j = i + 1; j < nr_dests; j++)
    do_send (skt, buff, bts, from_ll, j);
}

void
do_bridge (int rx_skt)
{
#define BUFF_SIZE	600
  int i, j;
  char buff[BUFF_SIZE];
  int n_num = 0, hd_len;
  char tmp[64];
  int bts;
  struct iph *iph;
  struct udph *udph;
  struct sockaddr_ll from;
  int from_len = sizeof (from);

  iph = (struct iph *) buff;
  udph = (struct udph *) ((void *) iph + sizeof (struct iph));

  for (;;)
    {
      bts = recvfrom (rx_skt, buff, BUFF_SIZE, 0, (struct sockaddr *) &from,
		      &from_len);

      if (iph->protocol != PROTUDP)
	continue;

/* from.sll_plttype != PACKET_BROADCAST would be enough for ethernet,
but we use ppp, too. Have to compare destination address... */
      if (((unsigned int *) iph->dst)[0] != 0xFFFFFFFF)
	continue;

      if (ntohs(udph->dst_port) < 1024)	/* do not forward reserved ports */
	continue;

      distribute (rx_skt, buff, bts, from);
    }
}

int
getifindex (int skt, char *str)
{
  struct ifreq irq;

  strcpy (irq.ifr_name, str);

  if (ioctl (skt, SIOCGIFINDEX, &irq) < 0)
    {
      perror ("SIOCGIFINDEX");
      printf ("ifname='%s'\n", str);
      return -1;
    }

  dests[nr_dests] = irq.ifr_ifindex;
  ifnames[nr_dests] = strdup (str);
  nr_dests++;

  return 0;
}

/* rx/tx socket */
int skt;

void
sighandler (int signr)
{
  int i;

  switch (signr)
    {
    case SIGINT:
    case SIGTERM:
      printf ("terminating udp bridge..\n");
      close (skt);

      for (i = 0; i < nr_dests; i++)
	free (ifnames[i]);

      exit (0);
      break;
    }
}

char *cfg_file = "/etc/udpbridge.conf";

int
parseconfig ()
{
  FILE *f;
  char *line;
#define BFLEN	256
  char buff[BFLEN];

  f = fopen (cfg_file, "r");
  if (f == NULL)
    {
      printf ("Cannot open cfg file '%s'\n", cfg_file);
      return -1;
    }

  while ((line = fgets (buff, BFLEN, f)) != NULL)
    {
      char *tmp;

      tmp = strchr (line, '\n');
      if (*tmp == '#' || *tmp == ';')
	continue;

      if (tmp != NULL)
	*tmp = '\0';

      getifindex (skt, line);
    }

  return 0;
}

void usage()
{
printf("usage: udpbridge [-f configfile]\n");
}

int
main (int argc, char **argv)
{
  int i, arg;

  printf ("UDP Bridge (c) 2002 Ondra Havel <xixo@bigfoot.com>\n");

  skt = socket (PF_PACKET, SOCK_DGRAM, htons (ETH_P_ALL));
  if (skt < 0)
    {
      perror ("socket (rx)");
      return -1;
    }

  while ((arg = getopt (argc, argv, "f:")) != EOF)
    {
      switch (arg)
	{
	case 'f':
	  cfg_file = optarg;
	  break;
	case '?':
	  usage();
	  close(skt);
	  return -2;
	}
    }

  if (parseconfig ())
    {
      close (skt);
      return -1;
    }

  signal (SIGINT, sighandler);
  signal (SIGTERM, sighandler);

  if (!nr_dests)
    {
      printf ("No interfaces found, terminating.\n");
      close (skt);
      return -2;
    }

  printf ("found %d interfaces: ", nr_dests);
  for (i = 0; i < nr_dests; i++)
    {
      printf (ifnames[i]);
      putchar (' ');
    }
  putchar ('\n');

  do_bridge (skt);
}
